@@ -8,7 +8,7 @@ from typing import Any, Dict
 
 import cloudscraper
 
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 class ApiClient:
@@ -22,11 +22,14 @@ class ApiClient:
         """Return a new Client instance."""
         self.session = session
         self.baseurl = baseurl
+
         if headers:
             self.headers = headers
         else:
             self.headers = self.default_headers.copy()
-        self.headers.update(aditional_headers)
+
+        if aditional_headers:
+            self.headers.update(aditional_headers)
 
     def url(self, addurl=None):
         """Return the url for the API endpoint."""
@@ -43,6 +46,10 @@ class ApiClient:
         if aditional_headers:
             total_headers.update(aditional_headers)
         url = self.url(addurl)
+
+        logger.debug("URL: %s", url)
+        logger.debug("Headers: %s", total_headers)
+
         try:
             response = self.session.get(url, headers=total_headers, params=params)
             response.raise_for_status()
@@ -53,20 +60,9 @@ class ApiClient:
             if response.status_code == 401:
                 raise GarminConnectAuthenticationError("Authentication error") from err
             if response.status_code == 403:
-                raise GarminConnectConnectionError(f"Forbidden get url: {url}") from err
-            if response.status_code == 500:
-                raise GarminConnectConnectionError("Server error") from err
-            if response.status_code == 404:
-                raise GarminConnectConnectionError("Not found") from err
-            try:
-                resp = response.json()
-                error = resp["message"].json()
-            except AttributeError:
-                error = "Unknown"
+                raise GarminConnectConnectionError(f"Forbidden url: {url}") from err
 
-            raise GarminConnectConnectionError(
-                f"Unknown error {response.status_code} - {error}"
-            ) from err
+            raise GarminConnectConnectionError(err) from err
 
     def post(self, addurl, aditional_headers, params, data):
         """Make an API call using the POST method."""
@@ -74,6 +70,11 @@ class ApiClient:
         if aditional_headers:
             total_headers.update(aditional_headers)
         url = self.url(addurl)
+
+        logger.debug("URL: %s", url)
+        logger.debug("Headers: %s", total_headers)
+        logger.debug("Data: %s", total_headers)
+
         try:
             response = self.session.post(
                 url, headers=total_headers, params=params, data=data
@@ -86,20 +87,9 @@ class ApiClient:
             if response.status_code == 401:
                 raise GarminConnectAuthenticationError("Authentication error") from err
             if response.status_code == 403:
-                raise GarminConnectConnectionError(f"Forbidden post url: {url}") from err
-            if response.status_code == 500:
-                raise GarminConnectConnectionError("Server error") from err
-            if response.status_code == 404:
-                raise GarminConnectConnectionError("Not found") from err
-            try:
-                resp = response.json()
-                error = resp["message"].json()
-            except AttributeError:
-                error = "Unknown"
+                raise GarminConnectConnectionError(f"Forbidden url: {url}") from err
 
-            raise GarminConnectConnectionError(
-                f"Unknown error {response.status_code} - {error}"
-            ) from err
+            raise GarminConnectConnectionError(err) from err
 
 
 class Garmin:
@@ -152,6 +142,12 @@ class Garmin:
         self.garmin_connect_heartrates_daily_url = (
             "proxy/wellness-service/wellness/dailyHeartRate"
         )
+        self.garmin_connect_daily_respiration_url = (
+            "proxy/wellness-service/wellness/daily/respiration"
+        )
+        self.garmin_connect_daily_spo2_url = (
+            "proxy/wellness-service/wellness/daily/spo2"
+        )
         self.garmin_connect_activities = (
             "proxy/activitylist-service/activities/search/activities"
         )
@@ -242,17 +238,21 @@ class Garmin:
 
         found = re.search(r"name=\"_csrf\" value=\"(\w*)", response.text, re.M)
         if not found:
-            logger.error("_csrf not found: %s", response.status_code)
+            logger.error("_csrf not found  (%d)", response.status_code)
             return False
-        logger.debug("_csrf found (%s).", found.group(1))
+
+        csrf = found.group(1)
+        logger.debug("_csrf found: %s", csrf)
+        logger.debug("Referer found: %s", response.url)
 
         data = {
             "username": self.username,
             "password": self.password,
             "embed": "false",
-            "_csrf": found.group(1),
+            "_csrf": csrf,
         }
         post_headers = {
+            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:66.0) Gecko/20100101 Firefox/66.0",
             "Referer": response.url,
             "Content-Type": "application/x-www-form-urlencoded",
         }
@@ -370,6 +370,22 @@ class Garmin:
 
         return self.modern_rest_client.get(url).json()
 
+    def get_respiration_data(self, cdate: str) -> Dict[str, Any]:
+        """Return available respiration data 'cdate' format 'YYYY-mm-dd'."""
+
+        url = f"{self.garmin_connect_daily_respiration_url}/{cdate}"
+        logger.debug("Requesting respiration data with URL: %s", url)
+
+        return self.modern_rest_client.get(url).json()
+
+    def get_spo2_data(self, cdate: str) -> Dict[str, Any]:
+        """Return available SpO2 data 'cdate' format 'YYYY-mm-dd'."""
+
+        url = f"{self.garmin_connect_daily_spo2_url}/{cdate}"
+        logger.debug("Requesting SpO2 data with URL: %s", url)
+
+        return self.modern_rest_client.get(url).json()
+
     def get_personal_record(self) -> Dict[str, Any]:
         """Return personal records for current user."""
 
@@ -546,7 +562,7 @@ class Garmin:
     def logout(self):
         """Log user out of session."""
 
-        self.modern_rest_client.get(self.garmin_connect_logout).json()
+        self.modern_rest_client.get(self.garmin_connect_logout)
 
 
 class GarminConnectConnectionError(Exception):
